@@ -1,7 +1,6 @@
 """
 The transformer module is for performing cartographic transformations.
 """
-
 __all__ = [
     "transform",
     "itransform",
@@ -9,34 +8,131 @@ __all__ = [
     "TransformerGroup",
     "AreaOfInterest",
 ]
+import threading
 import warnings
+from abc import ABC, abstractmethod
 from array import array
+from dataclasses import dataclass
 from itertools import chain, islice
 from pathlib import Path
-from typing import Any, Iterable, Iterator, List, Optional, Tuple, Union
+from typing import Any, Iterable, Iterator, List, Optional, Tuple, Union, overload
 
 from pyproj import CRS
+from pyproj._compat import cstrencode
 from pyproj._crs import AreaOfUse, CoordinateOperation
-from pyproj._transformer import (  # noqa
+from pyproj._transformer import (  # noqa: F401 pylint: disable=unused-import
     AreaOfInterest,
     _Transformer,
     _TransformerGroup,
     proj_version_str,
 )
-from pyproj.compat import cstrencode
 from pyproj.datadir import get_user_data_dir
-from pyproj.enums import TransformDirection, WktVersion
+from pyproj.enums import ProjVersion, TransformDirection, WktVersion
 from pyproj.exceptions import ProjError
 from pyproj.sync import _download_resource_file
 from pyproj.utils import _convertback, _copytobuffer
 
 
+class TransformerMaker(ABC):
+    """
+    .. versionadded:: 3.1.0
+
+    Base class for generating new instances
+    of the Cython _Transformer class for
+    thread safety in the Transformer class.
+    """
+
+    @abstractmethod
+    def __call__(self) -> _Transformer:
+        """
+        Returns
+        -------
+        _Transformer
+        """
+        raise NotImplementedError
+
+
+@dataclass(frozen=True)
+class TransformerUnsafe(TransformerMaker):
+    """
+    .. versionadded:: 3.1.0
+
+    Returns the original Cython _Transformer
+    and is not thread-safe.
+    """
+
+    transformer: _Transformer
+
+    def __call__(self) -> _Transformer:
+        """
+        Returns
+        -------
+        _Transformer
+        """
+        return self.transformer
+
+
+@dataclass(frozen=True)
+class TransformerFromCRS(TransformerMaker):
+    """
+    .. versionadded:: 3.1.0
+
+    Generates a Cython _Transformer class from input CRS data.
+    """
+
+    crs_from: bytes
+    crs_to: bytes
+    always_xy: bool
+    area_of_interest: Optional[AreaOfInterest]
+    authority: Optional[str]
+    accuracy: Optional[str]
+    allow_ballpark: Optional[bool]
+
+    def __call__(self) -> _Transformer:
+        """
+        Returns
+        -------
+        _Transformer
+        """
+        return _Transformer.from_crs(
+            self.crs_from,
+            self.crs_to,
+            always_xy=self.always_xy,
+            area_of_interest=self.area_of_interest,
+            authority=self.authority,
+            accuracy=self.accuracy,
+            allow_ballpark=self.allow_ballpark,
+        )
+
+
+@dataclass(frozen=True)
+class TransformerFromPipeline(TransformerMaker):
+    """
+    .. versionadded:: 3.1.0
+
+    Generates a Cython _Transformer class from input pipeline data.
+    """
+
+    proj_pipeline: bytes
+
+    def __call__(self) -> _Transformer:
+        """
+        Returns
+        -------
+        _Transformer
+        """
+        return _Transformer.from_pipeline(self.proj_pipeline)
+
+
 class TransformerGroup(_TransformerGroup):
     """
     The TransformerGroup is a set of possible transformers from one CRS to another.
 
     .. versionadded:: 2.3.0
 
+    .. warning:: CoordinateOperation and Transformer objects
+                 returned are not thread-safe.
+
     From PROJ docs::
 
         The operations are sorted with the most relevant ones first: by
@@ -58,6 +154,7 @@ class TransformerGroup(_TransformerGroup):
         """Get all possible transformations from a :obj:`pyproj.crs.CRS`
         or input used to create one.
 
+        .. deprecated:: 3.1 skip_equivalent
 
         Parameters
         ----------
@@ -65,28 +162,34 @@ class TransformerGroup(_TransformerGroup):
             Projection of input data.
         crs_to: pyproj.crs.CRS or input used to create one
             Projection of output data.
-        skip_equivalent: bool, optional
-            If true, will skip the transformation operation if input and output
-            projections are equivalent. Default is false.
-        always_xy: bool, optional
+        skip_equivalent: bool, default=False
+            DEPRECATED: If true, will skip the transformation operation
+            if input and output projections are equivalent.
+        always_xy: bool, default=False
             If true, the transform method will accept as input and return as output
             coordinates using the traditional GIS order, that is longitude, latitude
             for geographic CRS and easting, northing for most projected CRS.
-            Default is false.
         area_of_interest: :class:`pyproj.transformer.AreaOfInterest`, optional
             The area of interest to help order the transformations based on the
             best operation for the area.
 
         """
+        if skip_equivalent:
+            warnings.warn(
+                "skip_equivalent is deprecated.",
+                DeprecationWarning,
+                stacklevel=2,
+            )
+
         super().__init__(
-            CRS.from_user_input(crs_from),
-            CRS.from_user_input(crs_to),
-            skip_equivalent=skip_equivalent,
+            CRS.from_user_input(crs_from)._crs,
+            CRS.from_user_input(crs_to)._crs,
             always_xy=always_xy,
             area_of_interest=area_of_interest,
         )
         for iii, transformer in enumerate(self._transformers):
-            self._transformers[iii] = Transformer(transformer)
+            # pylint: disable=unsupported-assignment-operation
+            self._transformers[iii] = Transformer(TransformerUnsafe(transformer))
 
     @property
     def transformers(self) -> List["Transformer"]:
@@ -124,20 +227,24 @@ class TransformerGroup(_TransformerGroup):
 
         Download missing grids that can be downloaded automatically.
 
+        .. warning:: There are cases where the URL to download the grid is missing.
+                     In those cases, you can enable enable
+                     :ref:`debugging-internal-proj` and perform a
+                     transformation. The logs will show the grids PROJ searches for.
+
         Parameters
         ----------
         directory: str or Path, optional
             The directory to download the grids to.
             Defaults to :func:`pyproj.datadir.get_user_data_dir`
-        open_license: bool, optional
+        open_license: bool, default=True
             If True, will only download grids with an open license.
-            Defaults to False.
-        verbose: bool, optional
+        verbose: bool, default=False
             If True, will print information about grids downloaded.
-            Default is False.
         """
         if directory is None:
             directory = get_user_data_dir(True)
+        # pylint: disable=not-an-iterable
         for unavailable_operation in self.unavailable_operations:
             for grid in unavailable_operation.grids:
                 if (
@@ -163,6 +270,19 @@ class TransformerGroup(_TransformerGroup):
         )
 
 
+class TransformerLocal(threading.local):
+    """
+    Threading local instance for cython _Transformer class.
+
+    For more details, see:
+    https://github.com/pyproj4/pyproj/issues/782
+    """
+
+    def __init__(self):
+        self.transformer = None  # Initialises in each thread
+        super().__init__()
+
+
 class Transformer:
     """
     The Transformer class is for facilitating re-using
@@ -175,14 +295,33 @@ class Transformer:
 
     """
 
-    def __init__(self, base_transformer: Optional[_Transformer] = None) -> None:
-        if not isinstance(base_transformer, _Transformer):
+    def __init__(
+        self,
+        transformer_maker: Union[TransformerMaker, None] = None,
+    ) -> None:
+        if not isinstance(transformer_maker, TransformerMaker):
             ProjError.clear()
             raise ProjError(
                 "Transformer must be initialized using: "
                 "'from_crs', 'from_pipeline', or 'from_proj'."
             )
-        self._transformer = base_transformer
+
+        self._local = TransformerLocal()
+        self._local.transformer = transformer_maker()
+        self._transformer_maker = transformer_maker
+
+    @property
+    def _transformer(self):
+        """
+        The Cython _Transformer object for this thread.
+
+        Returns
+        -------
+        _Transformer
+        """
+        if self._local.transformer is None:
+            self._local.transformer = self._transformer_maker()
+        return self._local.transformer
 
     @property
     def name(self) -> str:
@@ -277,6 +416,38 @@ class Transformer:
         """
         return self._transformer.is_network_enabled
 
+    @property
+    def source_crs(self) -> Optional[CRS]:
+        """
+        .. versionadded:: 3.3.0
+
+        Returns
+        -------
+        Optional[CRS]:
+            The source CRS of a CoordinateOperation.
+        """
+        return (
+            None
+            if self._transformer.source_crs is None
+            else CRS(self._transformer.source_crs)
+        )
+
+    @property
+    def target_crs(self) -> Optional[CRS]:
+        """
+        .. versionadded:: 3.3.0
+
+        Returns
+        -------
+        Optional[CRS]:
+            The target CRS of a CoordinateOperation.
+        """
+        return (
+            None
+            if self._transformer.target_crs is None
+            else CRS(self._transformer.target_crs)
+        )
+
     @staticmethod
     def from_proj(
         proj_from: Any,
@@ -290,6 +461,7 @@ class Transformer:
         .. versionadded:: 2.1.2 skip_equivalent
         .. versionadded:: 2.2.0 always_xy
         .. versionadded:: 2.3.0 area_of_interest
+        .. deprecated:: 3.1 skip_equivalent
 
         Parameters
         ----------
@@ -297,14 +469,13 @@ class Transformer:
             Projection of input data.
         proj_to: :obj:`pyproj.Proj` or input used to create one
             Projection of output data.
-        skip_equivalent: bool, optional
-            If true, will skip the transformation operation if input and output
-            projections are equivalent. Default is false.
-        always_xy: bool, optional
+        skip_equivalent: bool, default=False
+            DEPRECATED: If true, will skip the transformation operation
+            if input and output projections are equivalent.
+        always_xy: bool, default=False
             If true, the transform method will accept as input and return as output
             coordinates using the traditional GIS order, that is longitude, latitude
             for geographic CRS and easting, northing for most projected CRS.
-            Default is false.
         area_of_interest: :class:`pyproj.transformer.AreaOfInterest`, optional
             The area of interest to help select the transformation.
 
@@ -313,6 +484,7 @@ class Transformer:
         Transformer
 
         """
+        # pylint: disable=import-outside-toplevel
         from pyproj import Proj
 
         if not isinstance(proj_from, Proj):
@@ -335,12 +507,17 @@ class Transformer:
         skip_equivalent: bool = False,
         always_xy: bool = False,
         area_of_interest: Optional[AreaOfInterest] = None,
+        authority: Optional[str] = None,
+        accuracy: Optional[float] = None,
+        allow_ballpark: Optional[bool] = None,
     ) -> "Transformer":
         """Make a Transformer from a :obj:`pyproj.crs.CRS` or input used to create one.
 
         .. versionadded:: 2.1.2 skip_equivalent
         .. versionadded:: 2.2.0 always_xy
         .. versionadded:: 2.3.0 area_of_interest
+        .. versionadded:: 3.1.0 authority, accuracy, allow_ballpark
+        .. deprecated:: 3.1 skip_equivalent
 
         Parameters
         ----------
@@ -348,29 +525,52 @@ class Transformer:
             Projection of input data.
         crs_to: pyproj.crs.CRS or input used to create one
             Projection of output data.
-        skip_equivalent: bool, optional
-            If true, will skip the transformation operation if input and output
-            projections are equivalent. Default is false.
-        always_xy: bool, optional
+        skip_equivalent: bool, default=False
+            DEPRECATED: If true, will skip the transformation operation
+            if input and output projections are equivalent.
+        always_xy: bool, default=False
             If true, the transform method will accept as input and return as output
             coordinates using the traditional GIS order, that is longitude, latitude
             for geographic CRS and easting, northing for most projected CRS.
-            Default is false.
         area_of_interest: :class:`pyproj.transformer.AreaOfInterest`, optional
             The area of interest to help select the transformation.
+        authority: str, optional
+            When not specified, coordinate operations from any authority will be
+            searched, with the restrictions set in the
+            authority_to_authority_preference database table related to the
+            authority of the source/target CRS themselves. If authority is set
+            to “any”, then coordinate operations from any authority will be
+            searched. If authority is a non-empty string different from "any",
+            then coordinate operations will be searched only in that authority
+            namespace (e.g. EPSG).
+        accuracy: float, optional
+            The minimum desired accuracy (in metres) of the candidate
+            coordinate operations.
+        allow_ballpark: bool, optional
+            Set to False to disallow the use of Ballpark transformation
+            in the candidate coordinate operations. Default is to allow.
 
         Returns
         -------
         Transformer
 
         """
+        if skip_equivalent:
+            warnings.warn(
+                "skip_equivalent is deprecated.",
+                DeprecationWarning,
+                stacklevel=2,
+            )
+
         return Transformer(
-            _Transformer.from_crs(
-                CRS.from_user_input(crs_from),
-                CRS.from_user_input(crs_to),
-                skip_equivalent=skip_equivalent,
+            TransformerFromCRS(
+                cstrencode(CRS.from_user_input(crs_from).srs),
+                cstrencode(CRS.from_user_input(crs_to).srs),
                 always_xy=always_xy,
                 area_of_interest=area_of_interest,
+                authority=authority,
+                accuracy=accuracy if accuracy is None else str(accuracy),
+                allow_ballpark=allow_ballpark,
             )
         )
 
@@ -378,7 +578,22 @@ class Transformer:
     def from_pipeline(proj_pipeline: str) -> "Transformer":
         """Make a Transformer from a PROJ pipeline string.
 
-        https://proj.org/operations/pipeline.html
+        :ref:`pipeline`
+
+        .. versionadded:: 3.1.0 AUTH:CODE string suppor (e.g. EPSG:1671)
+
+        Allowed input:
+          - a PROJ string
+          - a WKT string
+          - a PROJJSON string
+          - an object code (e.g. "EPSG:1671"
+            "urn:ogc:def:coordinateOperation:EPSG::1671")
+          - an object name. e.g "ITRF2014 to ETRF2014 (1)".
+            In that case as uniqueness is not guaranteed,
+            heuristics are applied to determine the appropriate best match.
+          - a OGC URN combining references for concatenated operations
+            (e.g. "urn:ogc:def:coordinateOperation,coordinateOperation:EPSG::3895,
+            coordinateOperation:EPSG::1618")
 
         Parameters
         ----------
@@ -390,23 +605,64 @@ class Transformer:
         Transformer
 
         """
-        return Transformer(_Transformer.from_pipeline(cstrencode(proj_pipeline)))
+        return Transformer(TransformerFromPipeline(cstrencode(proj_pipeline)))
 
-    def transform(
+    @overload
+    def transform(  # pylint: disable=invalid-name
         self,
         xx: Any,
         yy: Any,
-        zz: Any = None,
-        tt: Any = None,
         radians: bool = False,
         errcheck: bool = False,
         direction: Union[TransformDirection, str] = TransformDirection.FORWARD,
-    ) -> Any:
+        inplace: bool = False,
+    ) -> Tuple[Any, Any]:
+        ...
+
+    @overload
+    def transform(  # pylint: disable=invalid-name
+        self,
+        xx: Any,
+        yy: Any,
+        zz: Any,
+        radians: bool = False,
+        errcheck: bool = False,
+        direction: Union[TransformDirection, str] = TransformDirection.FORWARD,
+        inplace: bool = False,
+    ) -> Tuple[Any, Any, Any]:
+        ...
+
+    @overload
+    def transform(  # pylint: disable=invalid-name
+        self,
+        xx: Any,
+        yy: Any,
+        zz: Any,
+        tt: Any,
+        radians: bool = False,
+        errcheck: bool = False,
+        direction: Union[TransformDirection, str] = TransformDirection.FORWARD,
+        inplace: bool = False,
+    ) -> Tuple[Any, Any, Any, Any]:
+        ...
+
+    def transform(  # pylint: disable=invalid-name
+        self,
+        xx,
+        yy,
+        zz=None,
+        tt=None,
+        radians=False,
+        errcheck=False,
+        direction=TransformDirection.FORWARD,
+        inplace=False,
+    ):
         """
         Transform points between two coordinate systems.
 
         .. versionadded:: 2.1.1 errcheck
         .. versionadded:: 2.2.0 direction
+        .. versionadded:: 3.2.0 inplace
 
         Parameters
         ----------
@@ -418,20 +674,24 @@ class Transformer:
             Input z coordinate(s).
         tt: scalar or array (numpy or python), optional
             Input time coordinate(s).
-        radians: boolean, optional
+        radians: bool, default=False
             If True, will expect input data to be in radians and will return radians
-            if the projection is geographic. Default is False (degrees). Ignored for
-            pipeline transformations with pyproj 2, but will work in pyproj 3.
-        errcheck: boolean, optional
-            If True an exception is raised if the transformation is invalid.
-            By default errcheck=False and an invalid transformation
-            returns ``inf`` and no exception is raised.
+            if the projection is geographic. Otherwise, it uses degrees.
+            Ignored for pipeline transformations with pyproj 2,
+            but will work in pyproj 3.
+        errcheck: bool, default=False
+            If True, an exception is raised if the errors are found in the process.
+            If False, ``inf`` is returned for errors.
         direction: pyproj.enums.TransformDirection, optional
             The direction of the transform.
             Default is :attr:`pyproj.enums.TransformDirection.FORWARD`.
+        inplace: bool, default=False
+            If True, will attempt to write the results to the input array
+            instead of returning a new array. This will fail if the input
+            is not an array in C order with the double data type.
 
-
-        Example:
+        Example
+        --------
 
         >>> from pyproj import Transformer
         >>> transformer = Transformer.from_crs("epsg:4326", "epsg:3857")
@@ -467,21 +727,21 @@ class Transformer:
         >>> xpjr, ypjr, zpjr = transprojr.transform(xpj, ypj, zpj, radians=True)
         >>> f"{xpjr:.3f} {ypjr:.3f} {zpjr:.3f}"
         '-2704026.010 -4253051.810 3895878.820'
-        >>> transformer = Transformer.from_proj("epsg:4326", 4326, skip_equivalent=True)
+        >>> transformer = Transformer.from_proj("epsg:4326", 4326)
         >>> xeq, yeq = transformer.transform(33, 98)
         >>> f"{xeq:.0f}  {yeq:.0f}"
         '33  98'
 
         """
         # process inputs, making copies that support buffer API.
-        inx, xisfloat, xislist, xistuple = _copytobuffer(xx)
-        iny, yisfloat, yislist, yistuple = _copytobuffer(yy)
+        inx, x_data_type = _copytobuffer(xx, inplace=inplace)
+        iny, y_data_type = _copytobuffer(yy, inplace=inplace)
         if zz is not None:
-            inz, zisfloat, zislist, zistuple = _copytobuffer(zz)
+            inz, z_data_type = _copytobuffer(zz, inplace=inplace)
         else:
             inz = None
         if tt is not None:
-            intime, tisfloat, tislist, tistuple = _copytobuffer(tt)
+            intime, t_data_type = _copytobuffer(tt, inplace=inplace)
         else:
             intime = None
         # call pj_transform.  inx,iny,inz buffers modified in place.
@@ -495,17 +755,13 @@ class Transformer:
             errcheck=errcheck,
         )
         # if inputs were lists, tuples or floats, convert back.
-        outx = _convertback(xisfloat, xislist, xistuple, inx)
-        outy = _convertback(yisfloat, yislist, yistuple, iny)
-        return_data = (outx, outy)
+        outx = _convertback(x_data_type, inx)
+        outy = _convertback(y_data_type, iny)
+        return_data: Tuple[Any, ...] = (outx, outy)
         if inz is not None:
-            return_data += (  # type: ignore
-                _convertback(zisfloat, zislist, zistuple, inz),
-            )
+            return_data += (_convertback(z_data_type, inz),)
         if intime is not None:
-            return_data += (  # type: ignore
-                _convertback(tisfloat, tislist, tistuple, intime),
-            )
+            return_data += (_convertback(t_data_type, intime),)
         return return_data
 
     def itransform(
@@ -527,25 +783,26 @@ class Transformer:
         ----------
         points: list
             List of point tuples.
-        switch: boolean, optional
+        switch: bool, default=False
             If True x, y or lon,lat coordinates of points are switched to y, x
             or lat, lon. Default is False.
-        time_3rd: boolean, optional
+        time_3rd: bool, default=False
             If the input coordinates are 3 dimensional and the 3rd dimension is time.
-        radians: boolean, optional
+        radians: bool, default=False
             If True, will expect input data to be in radians and will return radians
-            if the projection is geographic. Default is False (degrees). Ignored for
-            pipeline transformations.
-        errcheck: boolean, optional
-            If True an exception is raised if the transformation is invalid.
-            By default errcheck=False and an invalid transformation
-            returns ``inf`` and no exception is raised.
+            if the projection is geographic. Otherwise, it uses degrees.
+            Ignored for pipeline transformations with pyproj 2,
+            but will work in pyproj 3.
+        errcheck: bool, default=False
+            If True, an exception is raised if the errors are found in the process.
+            If False, ``inf`` is returned for errors.
         direction: pyproj.enums.TransformDirection, optional
             The direction of the transform.
             Default is :attr:`pyproj.enums.TransformDirection.FORWARD`.
 
 
-        Example:
+        Example
+        --------
 
         >>> from pyproj import Transformer
         >>> transformer = Transformer.from_crs(4326, 2100)
@@ -588,19 +845,18 @@ class Transformer:
         ...     'EPSG:4326',
         ...     '+proj=longlat +datum=WGS84 +no_defs +type=crs',
         ...     always_xy=True,
-        ...     skip_equivalent=True
         ... )
         >>> for pt in transproj_eq.itransform([(-2.137, 0.661)]):
         ...     '{:.3f} {:.3f}'.format(*pt)
         '-2.137 0.661'
 
         """
-        it = iter(points)  # point iterator
+        point_it = iter(points)  # point iterator
         # get first point to check stride
         try:
-            fst_pt = next(it)
+            fst_pt = next(point_it)
         except StopIteration:
-            raise ValueError("iterable must contain at least one point")
+            raise ValueError("iterable must contain at least one point") from None
 
         stride = len(fst_pt)
         if stride not in (2, 3, 4):
@@ -611,7 +867,9 @@ class Transformer:
 
         # create a coordinate sequence generator etc. x1,y1,z1,x2,y2,z2,....
         # chain so the generator returns the first point that was already acquired
-        coord_gen = chain(fst_pt, (coords[c] for coords in it for c in range(stride)))
+        coord_gen = chain(
+            fst_pt, (coords[c] for coords in point_it for c in range(stride))
+        )
 
         while True:
             # create a temporary buffer storage for
@@ -630,14 +888,120 @@ class Transformer:
                 errcheck=errcheck,
             )
 
-            for pt in zip(*([iter(buff)] * stride)):
-                yield pt
+            for point in zip(*([iter(buff)] * stride)):
+                yield point
+
+    def transform_bounds(
+        self,
+        left: float,
+        bottom: float,
+        right: float,
+        top: float,
+        densify_pts: int = 21,
+        radians: bool = False,
+        errcheck: bool = False,
+        direction: Union[TransformDirection, str] = TransformDirection.FORWARD,
+    ) -> Tuple[float, float, float, float]:
+        """
+        .. versionadded:: 3.1.0
+
+        Transform boundary densifying the edges to account for nonlinear
+        transformations along these edges and extracting the outermost bounds.
+
+        If the destination CRS is geographic and right < left then the bounds
+        crossed the antimeridian. In this scenario there are two polygons,
+        one on each side of the antimeridian. The first polygon should be
+        constructed with (left, bottom, 180, top) and the second with
+        (-180, bottom, top, right).
+
+        To construct the bounding polygons with shapely::
+
+            def bounding_polygon(left, bottom, right, top):
+                if right < left:
+                    return shapely.geometry.MultiPolygon(
+                        [
+                            shapely.geometry.box(left, bottom, 180, top),
+                            shapely.geometry.box(-180, bottom, right, top),
+                        ]
+                    )
+                return shapely.geometry.box(left, bottom, right, top)
+
+
+        Parameters
+        ----------
+        left: float
+            Minimum bounding coordinate of the first axis in source CRS
+            (or the target CRS if using the reverse direction).
+        bottom: float
+            Minimum bounding coordinate of the second axis in source CRS.
+            (or the target CRS if using the reverse direction).
+        right: float
+            Maximum bounding coordinate of the first axis in source CRS.
+            (or the target CRS if using the reverse direction).
+        top: float
+            Maximum bounding coordinate of the second axis in source CRS.
+            (or the target CRS if using the reverse direction).
+        densify_points: uint, default=21
+            Number of points to add to each edge to account for nonlinear edges
+            produced by the transform process. Large numbers will produce worse
+            performance.
+        radians: bool, default=False
+            If True, will expect input data to be in radians and will return radians
+            if the projection is geographic. Otherwise, it uses degrees.
+        errcheck: bool, default=False
+            If True, an exception is raised if the errors are found in the process.
+            If False, ``inf`` is returned for errors.
+        direction: pyproj.enums.TransformDirection, optional
+            The direction of the transform.
+            Default is :attr:`pyproj.enums.TransformDirection.FORWARD`.
+
+        Returns
+        -------
+        left, bottom, right, top: float
+            Outermost coordinates in target coordinate reference system.
+        """
+        return self._transformer._transform_bounds(
+            left=left,
+            bottom=bottom,
+            right=right,
+            top=top,
+            densify_pts=densify_pts,
+            radians=radians,
+            errcheck=errcheck,
+            direction=direction,
+        )
+
+    def to_proj4(
+        self,
+        version: Union[ProjVersion, str] = ProjVersion.PROJ_5,
+        pretty: bool = False,
+    ) -> str:
+        """
+        Convert the projection to a PROJ string.
+
+        .. versionadded:: 3.1.0
+
+        Parameters
+        ----------
+        version: pyproj.enums.ProjVersion
+            The version of the PROJ string output.
+            Default is :attr:`pyproj.enums.ProjVersion.PROJ_5`.
+        pretty: bool, default=False
+            If True, it will set the output to be a multiline string.
+
+        Returns
+        -------
+        str:
+            The PROJ string.
+
+        """
+        return self._transformer.to_proj4(version=version, pretty=pretty)
 
     def to_wkt(
         self,
         version: Union[WktVersion, str] = WktVersion.WKT2_2019,
         pretty: bool = False,
-    ):
+    ) -> str:
         """
         Convert the projection to a WKT string.
 
@@ -652,11 +1016,11 @@ class Transformer:
 
         Parameters
         ----------
-        version: pyproj.enums.WktVersion
+        version: pyproj.enums.WktVersion, optional
             The version of the WKT output.
             Default is :attr:`pyproj.enums.WktVersion.WKT2_2019`.
-        pretty: bool
-            If True, it will set the output to be a multiline string. Defaults to False.
+        pretty: bool, default=False
+            If True, it will set the output to be a multiline string.
 
         Returns
         -------
@@ -673,10 +1037,10 @@ class Transformer:
 
         Parameters
         ----------
-        pretty: bool
-            If True, it will set the output to be a multiline string. Defaults to False.
-        indentation: int
-            If pretty is True, it will set the width of the indentation. Default is 2.
+        pretty: bool, default=False
+            If True, it will set the output to be a multiline string.
+        indentation: int, default=2
+            If pretty is True, it will set the width of the indentation.
 
         Returns
         -------
@@ -714,12 +1078,24 @@ class Transformer:
         return self._transformer.__eq__(other._transformer)
 
     def is_exact_same(self, other: Any) -> bool:
+        """
+        Check if the Transformer objects are the exact same.
+        If it is not a Transformer, then it returns False.
+
+        Parameters
+        ----------
+        other: Any
+
+        Returns
+        -------
+        bool
+        """
         if not isinstance(other, Transformer):
             return False
         return self._transformer.is_exact_same(other._transformer)
 
 
-def transform(
+def transform(  # pylint: disable=invalid-name
     p1: Any,
     p2: Any,
     x: Any,
@@ -734,8 +1110,10 @@ def transform(
     """
     .. versionadded:: 2.1.2 skip_equivalent
     .. versionadded:: 2.2.0 always_xy
+    .. deprecated::3.1 skip_equivalent
 
-    .. warning:: This function is deprecated. See: :ref:`upgrade_transformer`
+    .. deprecated:: 2.6.1
+        This function is deprecated. See: :ref:`upgrade_transformer`
 
     x2, y2, z2 = transform(p1, p2, x1, y1, z1)
 
@@ -754,12 +1132,10 @@ def transform(
     exception is raised if the transformation is
     invalid. By default errcheck=False and ``inf`` is returned for an
     invalid transformation (and no exception is raised).
-    If the optional kwarg skip_equivalent is true (default is False),
-    it will skip the transformation operation if input and output
-    projections are equivalent. If `always_xy` is toggled, the
-    transform method will accept as input and return as output
-    coordinates using the traditional GIS order, that is longitude, latitude
-    for geographic CRS and easting, northing for most projected CRS.
+    If `always_xy` is toggled, the transform method will accept as
+    input and return as output coordinates using the traditional GIS order,
+    that is longitude, latitude for geographic CRS and easting,
+    northing for most projected CRS.
 
     In addition to converting between cartographic and geographic
     projection coordinates, this function can take care of datum
@@ -820,7 +1196,7 @@ def transform(
     ).transform(xx=x, yy=y, zz=z, tt=tt, radians=radians, errcheck=errcheck)
 
 
-def itransform(
+def itransform(  # pylint: disable=invalid-name
     p1: Any,
     p2: Any,
     points: Iterable[Iterable],
@@ -834,8 +1210,10 @@ def itransform(
     """
     .. versionadded:: 2.1.2 skip_equivalent
     .. versionadded:: 2.2.0 always_xy
+    .. deprecated::3.1 skip_equivalent
 
-    .. warning:: This function is deprecated. See: :ref:`upgrade_transformer`
+    .. deprecated:: 2.6.1
+        This function is deprecated. See: :ref:`upgrade_transformer`
 
     points2 = itransform(p1, p2, points1)
     Iterator/generator version of the function pyproj.transform.
@@ -862,12 +1240,10 @@ def itransform(
     exception is raised if the transformation is
     invalid. By default errcheck=False and ``inf`` is returned for an
     invalid transformation (and no exception is raised).
-    If the optional kwarg skip_equivalent is true (default is False),
-    it will skip the transformation operation if input and output
-    projections are equivalent. If `always_xy` is toggled, the
-    transform method will accept as input and return as output
-    coordinates using the traditional GIS order, that is longitude, latitude
-    for geographic CRS and easting, northing for most projected CRS.
+    If `always_xy` is toggled, the transform method will accept as
+    input and return as output coordinates using the traditional GIS order,
+    that is longitude, latitude for geographic CRS and easting, northing
+    for most projected CRS.
 
 
     Example usage:
@@ -885,7 +1261,7 @@ def itransform(
     '411050.470 4497928.574'
     '399060.236 4486978.710'
     '458553.243 4523045.485'
-    >>> for pt in itransform(4326, 4326, [(30, 60)], skip_equivalent=True):
+    >>> for pt in itransform(4326, 4326, [(30, 60)]):
     ...     '{:.0f} {:.0f}'.format(*pt)
     '30 60'
 
