@@ -1,9 +1,13 @@
+import concurrent.futures
 import json
+import platform
 
 import numpy
 import pytest
 
+import pyproj
 from pyproj import CRS
+from pyproj._crs import AuthorityMatchInfo
 from pyproj.crs import (
     CoordinateOperation,
     CoordinateSystem,
@@ -15,7 +19,7 @@ from pyproj.crs.enums import CoordinateOperationType, DatumType
 from pyproj.enums import ProjVersion, WktVersion
 from pyproj.exceptions import CRSError
 from pyproj.transformer import TransformerGroup
-from test.conftest import PROJ_GTE_8, get_wgs84_datum_name, grids_available
+from test.conftest import PROJ_GTE_82, RGF93toWSG84, assert_can_pickle, grids_available
 
 
 class CustomCRS(object):
@@ -79,6 +83,11 @@ def test_from_string():
         )
 
 
+def test_from_string__invalid():
+    with pytest.raises(CRSError, match="CRS input is not a string"):
+        CRS.from_string(4326)
+
+
 def test_initialize_projparams_with_kwargs():
     crs_mixed_args = CRS("+proj=utm +zone=10", ellps="WGS84")
     crs_positional = CRS("+proj=utm +zone=10 +ellps=WGS84")
@@ -136,6 +145,11 @@ def test_is_projected():
     assert CRS.from_user_input(wgs84_crs).is_projected is False
 
 
+def test_is_compound():
+    assert CRS("EPSG:4326+5773").is_compound
+    assert not CRS("EPSG:4326").is_compound
+
+
 def test_is_same_crs():
     crs1 = CRS("urn:ogc:def:crs:OGC::CRS84")
     crs2 = CRS("EPSG:3857")
@@ -201,7 +215,7 @@ def test_repr():
             "Area of Use:\n"
             "- name: World.\n"
             "- bounds: (-180.0, -90.0, 180.0, 90.0)\n"
-            f"Datum: {get_wgs84_datum_name()}\n"
+            "Datum: World Geodetic System 1984 ensemble\n"
             "- Ellipsoid: WGS 84\n"
             "- Prime Meridian: Greenwich\n"
         )
@@ -209,10 +223,7 @@ def test_repr():
 
 def test_repr__long():
     with pytest.warns(FutureWarning):
-        if PROJ_GTE_8:
-            wkt_str = 'GEOGCRS["WGS 84",ENSEMBLE["World Geodetic System 1'
-        else:
-            wkt_str = 'GEOGCRS["WGS 84",DATUM["World Geodetic System 1984'
+        wkt_str = 'GEOGCRS["WGS 84",ENSEMBLE["World Geodetic System 1'
         assert repr(CRS(CRS({"init": "EPSG:4326"}).to_wkt())) == (
             f"<Geographic 2D CRS: {wkt_str} ...>\n"
             "Name: WGS 84\n"
@@ -222,7 +233,7 @@ def test_repr__long():
             "Area of Use:\n"
             "- name: World.\n"
             "- bounds: (-180.0, -90.0, 180.0, 90.0)\n"
-            f"Datum: {get_wgs84_datum_name()}\n"
+            "Datum: World Geodetic System 1984 ensemble\n"
             "- Ellipsoid: WGS 84\n"
             "- Prime Meridian: Greenwich\n"
         )
@@ -238,7 +249,7 @@ def test_repr_epsg():
         "Area of Use:\n"
         "- name: World.\n"
         "- bounds: (-180.0, -90.0, 180.0, 90.0)\n"
-        f"Datum: {get_wgs84_datum_name()}\n"
+        "Datum: World Geodetic System 1984 ensemble\n"
         "- Ellipsoid: WGS 84\n"
         "- Prime Meridian: Greenwich\n"
     )
@@ -313,10 +324,7 @@ def test_epsg():
 def test_datum():
     datum = CRS.from_epsg(4326).datum
     assert "\n" in repr(datum)
-    if PROJ_GTE_8:
-        datum_wkt = 'ENSEMBLE["World Geodetic System 1984 ensemble"'
-    else:
-        datum_wkt = 'DATUM["World Geodetic System 1984"'
+    datum_wkt = 'ENSEMBLE["World Geodetic System 1984 ensemble"'
     assert repr(datum).startswith(datum_wkt)
     assert datum.to_wkt().startswith(datum_wkt)
     assert datum == datum
@@ -604,7 +612,7 @@ def test_coordinate_operation__from_authority():
         "urn:ogc:def:coordinateOperation:EPSG::1671",
         CoordinateOperation.from_epsg(1671),
         CoordinateOperation.from_epsg(1671).to_json_dict(),
-        "RGF93 to WGS 84 (1)",
+        RGF93toWSG84,
     ],
 )
 def test_coordinate_operation__from_user_input(user_input):
@@ -629,31 +637,37 @@ def test_coordinate_operation__from_authority__empty():
 
 
 def test_datum__from_epsg():
-    if PROJ_GTE_8:
+    if PROJ_GTE_82:
         datum_wkt = (
             'ENSEMBLE["World Geodetic System 1984 ensemble",'
-            'MEMBER["World Geodetic System 1984 (Transit)",'
-            'ID["EPSG",1166]],MEMBER["World Geodetic System 1984 (G730)",'
-            'ID["EPSG",1152]],MEMBER["World Geodetic System 1984 (G873)",'
-            'ID["EPSG",1153]],MEMBER["World Geodetic System 1984 (G1150)",'
-            'ID["EPSG",1154]],MEMBER["World Geodetic System 1984 (G1674)",'
-            'ID["EPSG",1155]],MEMBER["World Geodetic System 1984 (G1762)",'
-            'ID["EPSG",1156]],ELLIPSOID["WGS 84",6378137,298.257223563,'
-            'LENGTHUNIT["metre",1],ID["EPSG",7030]],'
-            'ENSEMBLEACCURACY[2.0],ID["EPSG",6326]]'
+            'MEMBER["World Geodetic System 1984 (Transit)",ID["EPSG",1166]],'
+            'MEMBER["World Geodetic System 1984 (G730)",ID["EPSG",1152]],'
+            'MEMBER["World Geodetic System 1984 (G873)",ID["EPSG",1153]],'
+            'MEMBER["World Geodetic System 1984 (G1150)",ID["EPSG",1154]],'
+            'MEMBER["World Geodetic System 1984 (G1674)",ID["EPSG",1155]],'
+            'MEMBER["World Geodetic System 1984 (G1762)",ID["EPSG",1156]],'
+            'MEMBER["World Geodetic System 1984 (G2139)",ID["EPSG",1309]],'
+            'ELLIPSOID["WGS 84",6378137,298.257223563,LENGTHUNIT["metre",1],'
+            'ID["EPSG",7030]],ENSEMBLEACCURACY[2.0],ID["EPSG",6326]]'
         )
     else:
         datum_wkt = (
-            'DATUM["World Geodetic System 1984",'
-            'ELLIPSOID["WGS 84",6378137,298.257223563,'
-            'LENGTHUNIT["metre",1]],ID["EPSG",6326]]'
+            'ENSEMBLE["World Geodetic System 1984 ensemble",'
+            'MEMBER["World Geodetic System 1984 (Transit)",ID["EPSG",1166]],'
+            'MEMBER["World Geodetic System 1984 (G730)",ID["EPSG",1152]],'
+            'MEMBER["World Geodetic System 1984 (G873)",ID["EPSG",1153]],'
+            'MEMBER["World Geodetic System 1984 (G1150)",ID["EPSG",1154]],'
+            'MEMBER["World Geodetic System 1984 (G1674)",ID["EPSG",1155]],'
+            'MEMBER["World Geodetic System 1984 (G1762)",ID["EPSG",1156]],'
+            'ELLIPSOID["WGS 84",6378137,298.257223563,LENGTHUNIT["metre",1],'
+            'ID["EPSG",7030]],ENSEMBLEACCURACY[2.0],ID["EPSG",6326]]'
         )
     assert Datum.from_epsg("6326").to_wkt() == datum_wkt
 
 
 def test_datum__from_authority():
     dt = Datum.from_authority("EPSG", 6326)
-    assert dt.name == get_wgs84_datum_name()
+    assert dt.name == "World Geodetic System 1984 ensemble"
 
 
 def test_datum__from_epsg__invalid():
@@ -671,9 +685,7 @@ def test_datum__from_authority__invalid():
     [
         6326,
         ("EPSG", 6326),
-        "urn:ogc:def:ensemble:EPSG::6326"
-        if PROJ_GTE_8
-        else "urn:ogc:def:datum:EPSG::6326",
+        "urn:ogc:def:ensemble:EPSG::6326",
         Datum.from_epsg(6326),
         Datum.from_epsg(6326).to_json_dict(),
         "World Geodetic System 1984",
@@ -894,20 +906,14 @@ def test_datum_equals():
 @pytest.mark.parametrize(
     "input_str",
     [
-        "urn:ogc:def:ensemble:EPSG::6326"
-        if PROJ_GTE_8
-        else "urn:ogc:def:datum:EPSG::6326",
+        "urn:ogc:def:ensemble:EPSG::6326",
         "World Geodetic System 1984",
     ],
 )
 def test_datum__from_string(input_str):
     dd = Datum.from_string(input_str)
-    if PROJ_GTE_8:
-        assert dd.name == "World Geodetic System 1984 ensemble"
-        assert dd.type_name == "Datum Ensemble"
-    else:
-        assert dd.name == "World Geodetic System 1984"
-        assert dd.type_name == "Geodetic Reference Frame"
+    assert dd.name == "World Geodetic System 1984 ensemble"
+    assert dd.type_name == "Datum Ensemble"
 
 
 @pytest.mark.parametrize(
@@ -932,7 +938,7 @@ def test_datum__from_string__type_name(input_str, type_name):
 )
 def test_datum__from_name(input_name):
     dd = Datum.from_name(input_name)
-    assert dd.name == get_wgs84_datum_name()
+    assert dd.name == "World Geodetic System 1984 ensemble"
 
 
 @pytest.mark.parametrize("auth_name", [None, "ESRI"])
@@ -990,8 +996,11 @@ def test_ellipsoid__from_string(input_str):
     "input_str,long_name",
     [
         ("Airy 1830", "Airy 1830"),
-        ("intl", "International 1909 (Hayford)"),
-        ("International 1909 (Hayford)", "International 1909 (Hayford)"),
+        ("intl", "International 1924 (Hayford 1909, 1910)"),
+        (
+            "International 1924 (Hayford 1909, 1910)",
+            "International 1924 (Hayford 1909, 1910)",
+        ),
     ],
 )
 def test_ellipsoid__from_name(input_str, long_name):
@@ -1054,11 +1063,11 @@ def test_coordinate_operation_equals():
 
 
 @pytest.mark.parametrize(
-    "input_str", ["urn:ogc:def:coordinateOperation:EPSG::1671", "RGF93 to WGS 84 (1)"]
+    "input_str", ["urn:ogc:def:coordinateOperation:EPSG::1671", RGF93toWSG84]
 )
 def test_coordinate_operation__from_string(input_str):
     co = CoordinateOperation.from_string(input_str)
-    assert co.name == "RGF93 to WGS 84 (1)"
+    assert co.name == RGF93toWSG84
 
 
 def test_coordinate_operation__from_name():
@@ -1153,7 +1162,7 @@ def test_from_authority__ignf():
 
 def test_ignf_authority_repr():
     assert repr(CRS.from_authority("IGNF", "ETRS89UTM28")).startswith(
-        "<Projected CRS: IGNF:ETRS89UTM28>"
+        "<Derived Projected CRS: IGNF:ETRS89UTM28>"
     )
 
 
@@ -1318,6 +1327,11 @@ def test_to_dict_from_dict():
         assert CRS.from_dict(cc.to_dict()).name == "unknown"
 
 
+def test_from_dict__invalid():
+    with pytest.raises(CRSError, match="CRS input is not a dict"):
+        CRS.from_dict(4326)
+
+
 @pytest.mark.parametrize(
     "class_type",
     [Datum, Ellipsoid, PrimeMeridian, CoordinateOperation, CoordinateSystem],
@@ -1401,3 +1415,123 @@ def test_numpy_bool_kwarg_true():
         proj="utm", zone=32, ellipsis="WGS84", datum="WGS84", units="m", south=south
     )
     assert "+south " in crs.srs
+
+
+@pytest.mark.skipif(
+    pyproj._datadir._USE_GLOBAL_CONTEXT, reason="Global Context not Threadsafe."
+)
+def test_crs_multithread():
+    # https://github.com/pyproj4/pyproj/issues/782
+    crs = CRS(4326)
+
+    def to_wkt(num):
+        return crs.to_wkt()
+
+    with concurrent.futures.ThreadPoolExecutor(max_workers=2) as executor:
+        for result in executor.map(to_wkt, range(10)):
+            pass
+
+
+@pytest.mark.skipif(
+    platform.python_implementation() != "CPython", reason="pypy process unstable."
+)
+def test_crs_multiprocess():
+    # https://github.com/pyproj4/pyproj/issues/933
+    with concurrent.futures.ProcessPoolExecutor(max_workers=2) as executor:
+        for result in executor.map(CRS, [4326 for _ in range(10)]):
+            pass
+
+
+def test_coordinate_operation__to_proj4():
+    operation = CoordinateOperation.from_string(
+        "+proj=pipeline +step +proj=axisswap +order=2,1 +step "
+        "+proj=unitconvert +xy_in=deg +xy_out=rad +step "
+        "+proj=webmerc +lat_0=0 +lon_0=0 +x_0=0 +y_0=0 +ellps=WGS84"
+    )
+    proj_string = operation.to_proj4()
+    assert "+proj=pipeline" in proj_string
+    assert "\n" not in proj_string
+
+
+def test_coordinate_operation__to_proj4__pretty():
+    operation = CoordinateOperation.from_string(
+        "+proj=pipeline +step +proj=axisswap +order=2,1 +step "
+        "+proj=unitconvert +xy_in=deg +xy_out=rad +step "
+        "+proj=webmerc +lat_0=0 +lon_0=0 +x_0=0 +y_0=0 +ellps=WGS84"
+    )
+    proj_string = operation.to_proj4(pretty=True)
+    assert "+proj=pipeline" in proj_string
+    assert "\n" in proj_string
+
+
+@pytest.mark.parametrize(
+    "crs_input",
+    [
+        "EPSG:4326",
+        "EPSG:2056",
+    ],
+)
+def test_to_3d(crs_input):
+    crs = CRS(crs_input)
+    assert len(crs.axis_info) == 2
+    crs_3d = crs.to_3d()
+    assert len(crs_3d.axis_info) == 3
+    vert_axis = crs_3d.axis_info[-1]
+    assert vert_axis.name == "Ellipsoidal height"
+    assert vert_axis.unit_name == "metre"
+    assert vert_axis.direction == "up"
+    assert crs_3d.to_3d() == crs_3d
+    assert crs_3d.name == crs.name
+
+
+def test_to_3d__name():
+    crs_3d = CRS("EPSG:2056").to_3d(name="TEST")
+    assert crs_3d.name == "TEST"
+
+
+def test_crs__pickle(tmp_path):
+    assert_can_pickle(CRS("epsg:4326"), tmp_path)
+
+
+def test_is_derived():
+    assert CRS(
+        "+proj=ob_tran +o_proj=longlat +o_lat_p=0 +o_lon_p=0 +lon_0=0"
+    ).is_derived
+    assert not CRS("+proj=latlon").is_derived
+
+
+def test_inheritance__from_methods():
+    class ChildCRS(CRS):
+        def new_method(self):
+            return 1
+
+    def assert_inheritance_valid(new_crs):
+        assert new_crs.new_method() == 1
+        assert isinstance(new_crs, ChildCRS)
+        assert isinstance(new_crs.geodetic_crs, ChildCRS)
+        assert isinstance(new_crs.source_crs, (type(None), ChildCRS))
+        assert isinstance(new_crs.target_crs, (type(None), ChildCRS))
+        assert isinstance(new_crs.to_3d(), ChildCRS)
+        for sub_crs in new_crs.sub_crs_list:
+            assert isinstance(sub_crs, ChildCRS)
+
+    assert_inheritance_valid(ChildCRS.from_epsg(4326))
+    assert_inheritance_valid(ChildCRS.from_string("EPSG:2056"))
+    with pytest.warns(FutureWarning):
+        assert_inheritance_valid(ChildCRS.from_proj4("+init=epsg:4328 +towgs84=0,0,0"))
+    assert_inheritance_valid(ChildCRS.from_user_input("EPSG:4326+5773"))
+    assert_inheritance_valid(ChildCRS.from_json(CRS(4326).to_json()))
+    assert_inheritance_valid(ChildCRS.from_json_dict(CRS(4326).to_json_dict()))
+    assert_inheritance_valid(ChildCRS.from_wkt(CRS(4326).to_wkt()))
+
+
+def test_list_authority():
+    assert CRS("+proj=utm +zone=15").list_authority() == [
+        AuthorityMatchInfo(auth_name="EPSG", code="32615", confidence=70)
+    ]
+
+
+def test_list_authority__multiple():
+    auth_list = CRS("+proj=longlat").list_authority()
+    assert AuthorityMatchInfo(auth_name="OGC", code="CRS84", confidence=70) in auth_list
+    assert AuthorityMatchInfo(auth_name="EPSG", code="4326", confidence=70) in auth_list
