@@ -1,11 +1,13 @@
+import numpy
 import pytest
 from numpy.testing import assert_almost_equal
+from packaging import version
 
 from pyproj import CRS
 from pyproj.crs import ProjectedCRS
 from pyproj.crs._cf1x8 import _try_list_if_string
 from pyproj.crs.coordinate_operation import (
-    LambertAzumuthalEqualAreaConversion,
+    LambertAzimuthalEqualAreaConversion,
     LambertCylindricalEqualAreaConversion,
     MercatorAConversion,
     OrthographicConversion,
@@ -16,7 +18,7 @@ from pyproj.crs.coordinate_operation import (
     VerticalPerspectiveConversion,
 )
 from pyproj.exceptions import CRSError
-from test.conftest import get_wgs84_datum_name
+from test.conftest import PROJ_LOOSE_VERSION
 
 
 def _to_dict(operation):
@@ -42,6 +44,30 @@ def _test_roundtrip(expected_cf, wkt_startswith):
     assert cf_dict == expected_cf
 
 
+def test_cf_from_numpy_dtypes():
+    cf = {
+        "grid_mapping_name": "lambert_conformal_conic",
+        "standard_parallel": numpy.array([60, 30], dtype="f4"),
+        "longitude_of_central_meridian": numpy.float32(0),
+        "latitude_of_projection_origin": numpy.int32(45),
+    }
+    crs = CRS.from_cf(cf)
+    with pytest.warns(UserWarning):
+        assert crs.to_dict() == {
+            "datum": "WGS84",
+            "lat_0": 45,
+            "lat_1": 60,
+            "lat_2": 30,
+            "lon_0": 0,
+            "no_defs": None,
+            "proj": "lcc",
+            "type": "crs",
+            "units": "m",
+            "x_0": 0,
+            "y_0": 0,
+        }
+
+
 def test_to_cf_transverse_mercator():
     crs = CRS(
         proj="tmerc",
@@ -59,11 +85,11 @@ def test_to_cf_transverse_mercator():
         "semi_major_axis": 6378388.0,
         "semi_minor_axis": crs.ellipsoid.semi_minor_metre,
         "inverse_flattening": 297.0,
-        "reference_ellipsoid_name": "International 1909 (Hayford)",
+        "reference_ellipsoid_name": "International 1924 (Hayford 1909, 1910)",
         "longitude_of_prime_meridian": 0.0,
         "prime_meridian_name": "Greenwich",
         "horizontal_datum_name": (
-            "Unknown based on International 1909 (Hayford) ellipsoid"
+            "Unknown based on International 1924 (Hayford 1909, 1910) ellipsoid"
         ),
         "towgs84": towgs84_test,
         "grid_mapping_name": "transverse_mercator",
@@ -135,7 +161,7 @@ def test_from_cf_transverse_mercator(towgs84_test):
         "semi_major_axis": 6378388.0,
         "semi_minor_axis": crs.ellipsoid.semi_minor_metre,
         "inverse_flattening": 297.0,
-        "reference_ellipsoid_name": "International 1909 (Hayford)",
+        "reference_ellipsoid_name": "International 1924 (Hayford 1909, 1910)",
         "longitude_of_prime_meridian": 0.0,
         "prime_meridian_name": "Greenwich",
         "grid_mapping_name": "transverse_mercator",
@@ -241,7 +267,7 @@ def test_cf_from_utm():
         "longitude_of_prime_meridian": 0.0,
         "prime_meridian_name": "Greenwich",
         "geographic_crs_name": "WGS 84",
-        "horizontal_datum_name": get_wgs84_datum_name(),
+        "horizontal_datum_name": "World Geodetic System 1984 ensemble",
         "projected_crs_name": "WGS 84 / UTM zone 15N",
         "grid_mapping_name": "transverse_mercator",
         "latitude_of_projection_origin": 0.0,
@@ -361,8 +387,8 @@ def test_cf_rotated_latlon():
         "proj": "ob_tran",
         "o_proj": "longlat",
         "o_lat_p": 32.5,
-        "o_lon_p": 170.0,
-        "lon_0": 0,
+        "o_lon_p": 0,
+        "lon_0": 350,
         "datum": "WGS84",
         "no_defs": None,
         "type": "crs",
@@ -374,8 +400,8 @@ def test_cf_rotated_latlon__grid():
         dict(
             grid_mapping_name="rotated_latitude_longitude",
             grid_north_pole_latitude=32.5,
-            grid_north_pole_longitude=170.0,
-            north_pole_grid_longitude=1.0,
+            grid_north_pole_longitude=1.0,
+            north_pole_grid_longitude=170.0,
         )
     )
     with pytest.warns(UserWarning):
@@ -385,13 +411,68 @@ def test_cf_rotated_latlon__grid():
         "o_proj": "longlat",
         "o_lat_p": 32.5,
         "o_lon_p": 170.0,
-        "lon_0": 1.0,
+        "lon_0": 181.0,
         "datum": "WGS84",
         "no_defs": None,
         "type": "crs",
     }
 
 
+def test_rotated_pole_to_cf():
+    rotated_pole_wkt = (
+        'GEOGCRS["undefined",\n'
+        '    BASEGEOGCRS["Unknown datum based upon the GRS 1980 ellipsoid",\n'
+        '        DATUM["Not specified (based on GRS 1980 ellipsoid)",\n'
+        '            ELLIPSOID["GRS 1980",6378137,298.257222101,\n'
+        '                LENGTHUNIT["metre",1]]],\n'
+        '        PRIMEM["Greenwich",0,\n'
+        '            ANGLEUNIT["degree",0.0174532925199433]]],\n'
+        '    DERIVINGCONVERSION["Pole rotation (netCDF CF convention)",\n'
+        '        METHOD["Pole rotation (netCDF CF convention)"],\n'
+        '        PARAMETER["Grid north pole latitude (netCDF CF '
+        'convention)",2,\n'
+        '            ANGLEUNIT["degree",0.0174532925199433,\n'
+        '                ID["EPSG",9122]]],\n'
+        '        PARAMETER["Grid north pole longitude (netCDF CF '
+        'convention)",3,\n'
+        '            ANGLEUNIT["degree",0.0174532925199433,\n'
+        '                ID["EPSG",9122]]],\n'
+        '        PARAMETER["North pole grid longitude (netCDF CF '
+        'convention)",4,\n'
+        '            ANGLEUNIT["degree",0.0174532925199433,\n'
+        '                ID["EPSG",9122]]]],\n'
+        "    CS[ellipsoidal,2],\n"
+        '        AXIS["geodetic latitude (Lat)",north,\n'
+        "            ORDER[1],\n"
+        '            ANGLEUNIT["degree",0.0174532925199433,\n'
+        '                ID["EPSG",9122]]],\n'
+        '        AXIS["geodetic longitude (Lon)",east,\n'
+        "            ORDER[2],\n"
+        '            ANGLEUNIT["degree",0.0174532925199433,\n'
+        '                ID["EPSG",9122]]]]'
+    )
+    crs = CRS(rotated_pole_wkt)
+    expected_cf = {
+        "semi_major_axis": 6378137.0,
+        "semi_minor_axis": 6356752.314140356,
+        "inverse_flattening": 298.257222101,
+        "reference_ellipsoid_name": "GRS 1980",
+        "longitude_of_prime_meridian": 0.0,
+        "prime_meridian_name": "Greenwich",
+        "geographic_crs_name": "undefined",
+        "grid_mapping_name": "rotated_latitude_longitude",
+        "grid_north_pole_latitude": 2.0,
+        "grid_north_pole_longitude": 3.0,
+        "north_pole_grid_longitude": 4.0,
+        "horizontal_datum_name": "Not specified (based on GRS 1980 ellipsoid)",
+    }
+    cf_dict = crs.to_cf()
+    assert cf_dict.pop("crs_wkt").startswith("GEOGCRS[")
+    assert cf_dict == expected_cf
+    # test roundtrip
+    _test_roundtrip(expected_cf, "GEOGCRS[")
+
+
 def test_cf_lambert_conformal_conic_1sp():
     crs = CRS.from_cf(
         dict(
@@ -661,13 +742,15 @@ def test_geos_crs_sweep():
 
 
 def test_geos_crs_fixed_angle_axis():
-    crs = CRS.from_cf(
-        dict(
-            grid_mapping_name="geostationary",
-            perspective_point_height=1,
-            fixed_angle_axis="y",
+    with pytest.warns(DeprecationWarning):
+        crs = CRS.from_cf(
+            dict(
+                grid_mapping_name="geostationary",
+                perspective_point_height=1,
+                fixed_angle_axis="y",
+            ),
+            errcheck=True,
         )
-    )
     expected_cf = {
         "semi_major_axis": 6378137.0,
         "semi_minor_axis": crs.ellipsoid.semi_minor_metre,
@@ -840,6 +923,12 @@ def test_osgb_1936():
             "Scale factor at natural origin"
         ],
     }
+    if PROJ_LOOSE_VERSION >= version.parse("8.0.1"):
+        expected_cf.update(
+            geographic_crs_name="OSGB36",
+            horizontal_datum_name="Ordnance Survey of Great Britain 1936",
+            projected_crs_name="OSGB36 / British National Grid",
+        )
     cf_dict = crs.to_cf()
     assert cf_dict.pop("crs_wkt").startswith("PROJCRS[")
     assert cf_dict == expected_cf
@@ -1092,7 +1181,7 @@ def test_azimuthal_equidistant():
 
 
 def test_lambert_azimuthal_equal_area():
-    crs = ProjectedCRS(conversion=LambertAzumuthalEqualAreaConversion(1, 2, 3, 4))
+    crs = ProjectedCRS(conversion=LambertAzimuthalEqualAreaConversion(1, 2, 3, 4))
     expected_cf = {
         "semi_major_axis": 6378137.0,
         "semi_minor_axis": crs.ellipsoid.semi_minor_metre,
@@ -1517,7 +1606,6 @@ def test_build_custom_datum__default_ellipsoid():
 def test_cartesian_cs():
     unit = {"type": "LinearUnit", "name": "US Survey Foot", "conversion_factor": 0.3048}
     cartesian_cs = {
-        "$schema": "https://proj.org/schemas/v0.2/projjson.schema.json",
         "type": "CoordinateSystem",
         "subtype": "Cartesian",
         "axis": [
@@ -1544,7 +1632,9 @@ def test_cartesian_cs():
         },
         cartesian_cs=cartesian_cs,
     )
-    assert crs.coordinate_system.to_json_dict() == cartesian_cs
+    json_dict = crs.coordinate_system.to_json_dict()
+    json_dict.pop("$schema")
+    assert json_dict == cartesian_cs
     # test coordinate system
     assert crs.cs_to_cf() == [
         {
@@ -1564,7 +1654,6 @@ def test_cartesian_cs():
 
 def test_ellipsoidal_cs():
     ellipsoidal_cs = {
-        "$schema": "https://proj.org/schemas/v0.2/projjson.schema.json",
         "type": "CoordinateSystem",
         "subtype": "ellipsoidal",
         "axis": [
@@ -1590,7 +1679,9 @@ def test_ellipsoidal_cs():
         ),
         ellipsoidal_cs=ellipsoidal_cs,
     )
-    assert crs.coordinate_system.to_json_dict() == ellipsoidal_cs
+    json_dict = crs.coordinate_system.to_json_dict()
+    json_dict.pop("$schema")
+    assert json_dict == ellipsoidal_cs
     # test coordinate system
     assert crs.cs_to_cf() == [
         {
@@ -1610,7 +1701,6 @@ def test_ellipsoidal_cs():
 
 def test_ellipsoidal_cs__from_name():
     ellipsoidal_cs = {
-        "$schema": "https://proj.org/schemas/v0.2/projjson.schema.json",
         "type": "CoordinateSystem",
         "subtype": "ellipsoidal",
         "axis": [
@@ -1632,7 +1722,9 @@ def test_ellipsoidal_cs__from_name():
         dict(grid_mapping_name="latitude_longitude", geographic_crs_name="WGS 84"),
         ellipsoidal_cs=ellipsoidal_cs,
     )
-    assert crs.coordinate_system.to_json_dict() == ellipsoidal_cs
+    json_dict = crs.coordinate_system.to_json_dict()
+    json_dict.pop("$schema")
+    assert json_dict == ellipsoidal_cs
     # test coordinate system
     assert crs.cs_to_cf() == [
         {
@@ -1653,7 +1745,6 @@ def test_ellipsoidal_cs__from_name():
 def test_export_compound_crs_cs():
     unit = {"type": "LinearUnit", "name": "US Survey Foot", "conversion_factor": 0.3048}
     cartesian_cs = {
-        "$schema": "https://proj.org/schemas/v0.2/projjson.schema.json",
         "type": "CoordinateSystem",
         "subtype": "Cartesian",
         "axis": [
@@ -1667,7 +1758,6 @@ def test_export_compound_crs_cs():
         ],
     }
     vertical_cs = {
-        "$schema": "https://proj.org/schemas/v0.2/projjson.schema.json",
         "type": "CoordinateSystem",
         "subtype": "vertical",
         "axis": [
@@ -1702,8 +1792,12 @@ def test_export_compound_crs_cs():
         cartesian_cs=cartesian_cs,
         vertical_cs=vertical_cs,
     )
-    assert crs.sub_crs_list[0].coordinate_system.to_json_dict() == cartesian_cs
-    assert crs.sub_crs_list[1].coordinate_system.to_json_dict() == vertical_cs
+    cartesian_json_dict = crs.sub_crs_list[0].coordinate_system.to_json_dict()
+    cartesian_json_dict.pop("$schema")
+    vertical_json_dict = crs.sub_crs_list[1].coordinate_system.to_json_dict()
+    vertical_json_dict.pop("$schema")
+    assert cartesian_json_dict == cartesian_cs
+    assert vertical_json_dict == vertical_cs
     # test coordinate system
     assert crs.cs_to_cf() == [
         {
