@@ -4,10 +4,12 @@ from numpy.testing import assert_almost_equal
 from pyproj.crs import GeographicCRS
 from pyproj.crs.coordinate_operation import (
     AlbersEqualAreaConversion,
+    AzimuthalEquidistantConversion,
     AzumuthalEquidistantConversion,
     EquidistantCylindricalConversion,
     GeostationarySatelliteConversion,
     HotineObliqueMercatorBConversion,
+    LambertAzimuthalEqualAreaConversion,
     LambertAzumuthalEqualAreaConversion,
     LambertConformalConic1SPConversion,
     LambertConformalConic2SPConversion,
@@ -19,6 +21,7 @@ from pyproj.crs.coordinate_operation import (
     PlateCarreeConversion,
     PolarStereographicAConversion,
     PolarStereographicBConversion,
+    PoleRotationNetCDFCFConversion,
     RotatedLatitudeLongitudeConversion,
     SinusoidalConversion,
     StereographicConversion,
@@ -28,6 +31,7 @@ from pyproj.crs.coordinate_operation import (
     VerticalPerspectiveConversion,
 )
 from pyproj.exceptions import CRSError
+from test.conftest import PROJ_GTE_82
 
 
 def _to_dict(operation):
@@ -74,8 +78,21 @@ def test_albers_equal_area_operation():
     }
 
 
+def test_azumuthal_equidistant_operation__defaults():
+    with pytest.warns(FutureWarning, match="deprecated"):
+        aeop = AzumuthalEquidistantConversion()
+    assert aeop.name == "unknown"
+    assert aeop.method_name == "Modified Azimuthal Equidistant"
+    assert _to_dict(aeop) == {
+        "Latitude of natural origin": 0.0,
+        "Longitude of natural origin": 0.0,
+        "False easting": 0.0,
+        "False northing": 0.0,
+    }
+
+
 def test_azimuthal_equidistant_operation__defaults():
-    aeop = AzumuthalEquidistantConversion()
+    aeop = AzimuthalEquidistantConversion()
     assert aeop.name == "unknown"
     assert aeop.method_name == "Modified Azimuthal Equidistant"
     assert _to_dict(aeop) == {
@@ -87,7 +104,7 @@ def test_azimuthal_equidistant_operation__defaults():
 
 
 def test_azimuthal_equidistant_operation():
-    aeop = AzumuthalEquidistantConversion(
+    aeop = AzimuthalEquidistantConversion(
         latitude_natural_origin=1,
         longitude_natural_origin=2,
         false_easting=3,
@@ -142,8 +159,21 @@ def test_geostationary_operation__invalid_sweep():
         GeostationarySatelliteConversion(sweep_angle_axis="P", satellite_height=10)
 
 
+def test_lambert_azumuthal_equal_area_operation__defaults():
+    with pytest.warns(FutureWarning, match="deprecated"):
+        aeop = LambertAzumuthalEqualAreaConversion()
+    assert aeop.name == "unknown"
+    assert aeop.method_name == "Lambert Azimuthal Equal Area"
+    assert _to_dict(aeop) == {
+        "Latitude of natural origin": 0.0,
+        "Longitude of natural origin": 0.0,
+        "False easting": 0.0,
+        "False northing": 0.0,
+    }
+
+
 def test_lambert_azimuthal_equal_area_operation__defaults():
-    aeop = LambertAzumuthalEqualAreaConversion()
+    aeop = LambertAzimuthalEqualAreaConversion()
     assert aeop.name == "unknown"
     assert aeop.method_name == "Lambert Azimuthal Equal Area"
     assert _to_dict(aeop) == {
@@ -155,7 +185,7 @@ def test_lambert_azimuthal_equal_area_operation__defaults():
 
 
 def test_lambert_azimuthal_equal_area_operation():
-    aeop = LambertAzumuthalEqualAreaConversion(
+    aeop = LambertAzimuthalEqualAreaConversion(
         latitude_natural_origin=1,
         longitude_natural_origin=2,
         false_easting=3,
@@ -612,6 +642,52 @@ def test_rotated_latitude_longitude_operation():
     assert _to_dict(aeaop) == {"o_lat_p": 1.0, "o_lon_p": 2.0, "lon_0": 3.0}
 
 
+def test_pole_rotation_netcdf_cf_convention__defaults():
+    poleop = PoleRotationNetCDFCFConversion(
+        grid_north_pole_latitude=1, grid_north_pole_longitude=2
+    )
+    if PROJ_GTE_82:
+        assert poleop.name == "Pole rotation (netCDF CF convention)"
+        assert poleop.method_name == "Pole rotation (netCDF CF convention)"
+        assert _to_dict(poleop) == {
+            "Grid north pole latitude (netCDF CF convention)": 1.0,
+            "Grid north pole longitude (netCDF CF convention)": 2.0,
+            "North pole grid longitude (netCDF CF convention)": 0.0,
+        }
+    else:
+        assert poleop.name == "unknown"
+        assert poleop.method_name == "PROJ ob_tran o_proj=longlat"
+        assert _to_dict(poleop) == {
+            "o_lat_p": 1.0,
+            "o_lon_p": 0.0,
+            "lon_0": 182.0,
+        }
+
+
+def test_pole_rotation_netcdf_cf_convention():
+    poleop = PoleRotationNetCDFCFConversion(
+        grid_north_pole_latitude=1,
+        grid_north_pole_longitude=2,
+        north_pole_grid_longitude=10,
+    )
+    if PROJ_GTE_82:
+        assert poleop.name == "Pole rotation (netCDF CF convention)"
+        assert poleop.method_name == "Pole rotation (netCDF CF convention)"
+        assert _to_dict(poleop) == {
+            "Grid north pole latitude (netCDF CF convention)": 1.0,
+            "Grid north pole longitude (netCDF CF convention)": 2.0,
+            "North pole grid longitude (netCDF CF convention)": 10.0,
+        }
+    else:
+        assert poleop.name == "unknown"
+        assert poleop.method_name == "PROJ ob_tran o_proj=longlat"
+        assert _to_dict(poleop) == {
+            "o_lat_p": 1.0,
+            "o_lon_p": 10.0,
+            "lon_0": 182.0,
+        }
+
+
 def test_lambert_cylindrical_equal_area_scale_operation__defaults():
     lceaop = LambertCylindricalEqualAreaScaleConversion()
     assert lceaop.name == "unknown"
